@@ -9,11 +9,12 @@
 #include "alkexception.h"
 
 AlkException::AlkException(const QString &msg, const QString &file, const unsigned long line)
+  : m_msg(msg)
+  , m_file(file)
+  , m_line(line)
+
 {
     // qDebug("ALKEXCEPTION(%s,%s,%ul)", qPrintable(msg), qPrintable(file), line);
-    m_msg = msg;
-    m_file = file;
-    m_line = line;
 }
 
 AlkException::~AlkException()
